          section   TEXT

          comment   C68 libfi2_a v1.20

          xdef      __usefi2
          xdef      __freefi2

* This sub-routine is to be called to use the "FileInfo II extensions" thing.
* It needs the extension Id in D2.L and returns the corresponding vector into
* A4. An error may be returned if the thing system is absent or if the thing
* is not found.
* Modified registers: D0 to D3, A0 to A2, A4.

__usefi2  LEA       FI2EXT,A0                     Thing name.
          BSR.S     USE_THING                     Use it.
          BNE.S     USEFI2END                     If error, return now.
          LEA       $18(A1),A4                    Vector for the extension.
USEFI2END TST.L     D0                            Set CCR accordingly to error.
          RTS                                     Return.

* This sub-routine is to be called to free the "FileInfo II extensions" thing.
* Modified registers: D1, D2, A0, A2.

__freefi2 MOVE.L    D0,-(SP)                      Keep previous error code.
          LEA       FI2EXT,A0                     Thing name.
          BSR.S     FREE_THG                      Free it.
          MOVE.L    (SP)+,D0                      Restore code and set CCR.
          RTS                                     Return.

FI2EXT    dc.w      22
          dc.b      'FileInfo II extensions'

********************************************************************************
* Routines d'utilisation et de libration d'une chose:
* ===================================================
*
* Paramtres en entre: D0.L = $28 ou $29 (si appel de DO_THING),
* ~~~~~~~~~~~~~~~~~~~~  D1.L = Id du job concern (si appel de USE_THG),
*                       D2.L = Paramtre supplmentaire ventuel (selon chose),
*                       D3.L = Paramtre supplmentaire ventuel (si FREE_THG),
*                       A0.L = Pointeur sur le nom de la chose,
*                       A1.L = Paramtre supplmentaire ventuel (FREE_THG),
*                       A2.L = Paramtre supplmentaire ventuel (selon chose),
*
* Paramtres en sortie: D0.L = Code d'erreur,
* ~~~~~~~~~~~~~~~~~~~~  D1.L = Id du job concern (ou -1 si appel de FREE_THG),
*                       D2.L = Rsultat (selon la chose),
*                       D3.L = Version de la chose (si SMS.UTHG) ou prserv,
*                       D4.L  D7.L: prservs,
*                       A1.L = Pointeur sur la chose (si SMS.UTHG) ou prserv,
*                       A2.L = Ptr/bloc chanage chose (si SMS.UTHG) ou rsultat
*                       A0.L, A3.L  A7.L: prservs.

FREE_THG  MOVEQ     #-1,D1                        Job utilisateur=job courant.
          MOVEQ     #$29,D0                       Code pour appel  SMS.FTHG.
          BRA.S     DO_THG

USE_THING MOVEQ     #-1,D1                        Job utilisateur=job courant.
USE_THG   MOVEQ     #$28,D0                       Code pour appel  SMS.UTHG.
DO_THING  MOVEQ     #-1,D3                        Temporisation infinie.
DO_THG    MOVE.L    A4,-(SP)                      | Sauvegarder le contexte.
          MOVEM.L   D0/A0,-(SP)                   |
          BSR.S     SEARCH                        Chercher le vecteur "THING".
          MOVEM.L   (SP)+,D0/A0                   Restaurer ptr nom est code.
          BNE.S     DONE1                         Si erreur pas d'appel.
          JSR       (A4)                          Appel au vecteur "THING".
DONE1     MOVEA.L   (SP)+,A4                      Restaurer contexte.
          CMPI.L    #-7,D0                        Erreur "not found" ?
          BNE.S     DONE2                         Si non, sauter ce qui suit...
          MOVEQ     #-19,D0                       Code erreur "not implemented".
DONE2     TST.L     D0                            Positionner CCR.
          RTS                                     Retour.

********************************************************************************
* Routine de recherche de la chose "THING":
* ========================================
*
* Paramtres en entre: nant
* ~~~~~~~~~~~~~~~~~~~~
* Paramtres en sortie: A0.L=Adresse de "THING" (si D0=0),
* ~~~~~~~~~~~~~~~~~~~~  A4.L=Adresse de la routine "TH_ENTRY",
*                       D0.L=Code erreur (0: pas d'erreur, -19: non implment),
*
* Registres modifis  : D0, A0, A4.
* ~~~~~~~~~~~~~~~~~~

SEARCH    MOVEM.L   D1/D2,-(SP)                   Sauver le contexte.
          MOVEQ     #0,D0                         | Prendre les informations
          TRAP      #1                            | sur le systme.
          MOVEQ     #-19,D0                       Code erreur "non implment".
          LEA       $B8(A0),A0                    Pointeur sur liste des choses.
          MOVE.L    A0,D1                         Pointeur nul ?
          BEQ.S     SEARCH3                       Si oui, retour (pas de chose).
SEARCH1   MOVE.L    (A0),D1                       D1=Adresse du linkage block.
          BEQ.S     SEARCH2                       Si dernire chose, fin
          MOVEA.L   D1,A0                         | sinon poursuivre la recher-
          BRA.S     SEARCH1                       | che avec la chose suivante.
SEARCH2   MOVEA.L   16(A0),A0                     Pointeur sur chose elle-mme.
          CMPI.L    #'THG%',(A0)                  Est-ce bien une chose ?
          BNE.S     SEARCH3                       Si non, retour avec erreur.
          CMPI.L    #-1,4(A0)                     Est-ce la chose "THING" ?
          BNE.S     SEARCH3                       Si non, retour avec erreur.
          MOVEA.L   8(A0),A4                      Prendre le vecteur TH_ENTRY.
          MOVEQ     #0,D0                         Pas d'erreur.
SEARCH3   MOVEM.L   (SP)+,D1/D2                   Restaurer le contexte.
          TST.L     D0                            Positionner CCR.
          RTS                                     Retour.

          end
